@@ -1,8 +1,8 @@
 /*
- *******************************************************************************
- * Copyright (C) 2007, Google, IBM and  *
- * others. All Rights Reserved. *
- *******************************************************************************
+ ********************************************************************************
+ * Copyright (C) 2007-2009, Google, International Business Machines Corporation *
+ * and others. All Rights Reserved.                                             *
+ ********************************************************************************
  */
 
 package com.ibm.icu.dev.test.format;
@@ -11,7 +11,6 @@ import java.text.ParseException;
 import java.text.ParsePosition;
 import java.util.Date;
 
-import com.ibm.icu.impl.ZoneMeta;
 import com.ibm.icu.lang.UCharacter;
 import com.ibm.icu.text.SimpleDateFormat;
 import com.ibm.icu.util.BasicTimeZone;
@@ -109,46 +108,89 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
                         tz.getOffset(DATES[datidx].getTime(), false, inOffsets);
                         outtz.getOffset(DATES[datidx].getTime(), false, outOffsets);
 
-                        // Check if localized GMT format or RFC format is used.
-                        int numDigits = 0;
-                        for (int n = 0; n < tzstr.length(); n++) {
-                            if (UCharacter.isDigit(tzstr.charAt(n))) {
-                                numDigits++;
-                            }
-                        }
-                        if (numDigits >= 4) {
-                            // Localized GMT or RFC: total offset (raw + dst) must be preserved.
-                            int inOffset = inOffsets[0] + inOffsets[1];
-                            int outOffset = outOffsets[0] + outOffsets[1];
-                            if (inOffset != outOffset) {
-                                errln("Offset round trip failed; tz=" + tzids[tzidx]
-                                    + ", locale=" + LOCALES[locidx] + ", pattern=" + PATTERNS[patidx]
-                                    + ", time=" + DATES[datidx].getTime() + ", str=" + tzstr
-                                    + ", inOffset=" + inOffset + ", outOffset=" + outOffset);
-                            }
-                        } else if (PATTERNS[patidx].equals("z") || PATTERNS[patidx].equals("zzzz")
-                                || PATTERNS[patidx].equals("v") || PATTERNS[patidx].equals("vvvv")
-                                || PATTERNS[patidx].equals("V")) {
-                            // Specific or generic: raw offset must be preserved.
-                            if (inOffsets[0] != outOffsets[0]) {
-                                errln("Raw offset round trip failed; tz=" + tzids[tzidx]
-                                    + ", locale=" + LOCALES[locidx] + ", pattern=" + PATTERNS[patidx]
-                                    + ", time=" + DATES[datidx].getTime() + ", str=" + tzstr
-                                    + ", inRawOffset=" + inOffsets[0] + ", outRawOffset=" + outOffsets[0]);
-                            }
-                        } else { // "VVVV"
-                            // Location: time zone rule must be preserved.
-                            if (!outtz.getID().equals(ZoneMeta.getCanonicalID(tzids[tzidx]))) {
+                        if (PATTERNS[patidx].equals("VVVV")) {
+                            // Location: time zone rule must be preserved except
+                            // zones not actually associated with a specific location.
+                            // Time zones in this category do not have "/" in its ID.
+                            String canonicalID = TimeZone.getCanonicalID(tzids[tzidx]);
+                            if (canonicalID != null && !outtz.getID().equals(canonicalID)) {
                                 // Canonical ID did not match - check the rules
-                                if (!((BasicTimeZone)outtz).hasEquivalentTransitions(tz, low, high)) {
+                                boolean bFailure = false;
+                                if ((tz instanceof BasicTimeZone) && (outtz instanceof BasicTimeZone)) {
+                                    bFailure = !(canonicalID.indexOf('/') == -1)
+                                                && !((BasicTimeZone)outtz).hasEquivalentTransitions(tz, low, high);
+                                }
+                                if (bFailure) {
+                                    // There are tzids not supported by CLDR.  For example,
+                                    // new tzids introduced in the tz database after CLDR release.
+                                    // In 4.0, we used to use GMT as fallback, but we changed the
+                                    // fallback behavior in later releases.
+                                    // This workaround contains list of tzids known to fail.
+                                    // There is not good way to identify such zones (no corresponding
+                                    // CLDR data) except for accessing the resource data directly.
+                                    final String[] excludedTZIDs = {
+                                        "America/Argentina/Salta", "America/Santarem",
+                                        "Asia/Riyadh87", "Asia/Riyadh88", "Asia/Riyadh89",
+                                    };
+                                    for (int i = 0; i < excludedTZIDs.length; i++) {
+                                        if (canonicalID.equals(excludedTZIDs[i])) {
+                                            bFailure = false;
+                                            break;
+                                        }
+                                    }
+                                }
+                                if (bFailure) {
                                     errln("Canonical round trip failed; tz=" + tzids[tzidx]
+                                            + ", locale=" + LOCALES[locidx] + ", pattern=" + PATTERNS[patidx]
+                                            + ", time=" + DATES[datidx].getTime() + ", str=" + tzstr
+                                            + ", outtz=" + outtz.getID());
+                                } else {
+                                    logln("Canonical round trip failed (as expected); tz=" + tzids[tzidx]
+                                            + ", locale=" + LOCALES[locidx] + ", pattern=" + PATTERNS[patidx]
+                                            + ", time=" + DATES[datidx].getTime() + ", str=" + tzstr
+                                            + ", outtz=" + outtz.getID());
+                                }
+                            }
+                        } else {
+                            // Check if localized GMT format or RFC format is used.
+                            int numDigits = 0;
+                            for (int n = 0; n < tzstr.length(); n++) {
+                                if (UCharacter.isDigit(tzstr.charAt(n))) {
+                                    numDigits++;
+                                }
+                            }
+
+                            if (numDigits >= 3) {
+                                // Localized GMT or RFC: total offset (raw + dst) must be preserved.
+                                int inOffset = inOffsets[0] + inOffsets[1];
+                                int outOffset = outOffsets[0] + outOffsets[1];
+                                if (inOffset != outOffset) {
+                                    errln("Offset round trip failed; tz=" + tzids[tzidx]
                                         + ", locale=" + LOCALES[locidx] + ", pattern=" + PATTERNS[patidx]
                                         + ", time=" + DATES[datidx].getTime() + ", str=" + tzstr
-                                        + ", outtz=" + outtz.getID());
+                                        + ", inOffset=" + inOffset + ", outOffset=" + outOffset);
+                                }
+                            } else {
+                                // Specific or generic: raw offset must be preserved.
+                                if (inOffsets[0] != outOffsets[0]) {
+                                    if (TimeZone.getDefaultTimeZoneType() == TimeZone.TIMEZONE_JDK
+                                            && tzids[tzidx].startsWith("SystemV/")) {
+                                        // JDK uses rule SystemV for these zones while
+                                        // ICU handles these zones as aliases of existing time zones
+                                        logln("Raw offset round trip failed; tz=" + tzids[tzidx]
+                                            + ", locale=" + LOCALES[locidx] + ", pattern=" + PATTERNS[patidx]
+                                            + ", time=" + DATES[datidx].getTime() + ", str=" + tzstr
+                                            + ", inRawOffset=" + inOffsets[0] + ", outRawOffset=" + outOffsets[0]);
+
+                                    } else {
+                                        errln("Raw offset round trip failed; tz=" + tzids[tzidx]
+                                            + ", locale=" + LOCALES[locidx] + ", pattern=" + PATTERNS[patidx]
+                                            + ", time=" + DATES[datidx].getTime() + ", str=" + tzstr
+                                            + ", inRawOffset=" + inOffsets[0] + ", outRawOffset=" + outOffsets[0]);
+                                    }
                                 }
                             }
                         }
-                        
                     }
                 }
             }
@@ -162,12 +204,25 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
      * round trip as expected.
      */
     public void TestTimeRoundTrip() {
-        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
 
-        cal.set(1900, Calendar.JANUARY, 1);
+        //boolean TEST_ALL = "true".equalsIgnoreCase(getProperty("TimeZoneRoundTripAll"));
+        boolean TEST_ALL = false;
+
+        int startYear, endYear;
+
+        if (TEST_ALL || getInclusion() > 5) {
+            startYear = 1900;
+        } else {
+            startYear = 1990;
+        }
+
+        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
+        endYear = cal.get(Calendar.YEAR) + 3;
+
+        cal.set(startYear, Calendar.JANUARY, 1);
         final long START_TIME = cal.getTimeInMillis();
 
-        cal.set(2020, Calendar.JANUARY, 1);
+        cal.set(endYear, Calendar.JANUARY, 1);
         final long END_TIME = cal.getTimeInMillis();
 
         // Whether each pattern is ambiguous at DST->STD local time overlap
@@ -178,14 +233,13 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
         final String BASEPATTERN = "yyyy-MM-dd'T'HH:mm:ss.SSS";
 
         ULocale[] LOCALES = null;
-        boolean DEBUG_ALL = false;
         boolean REALLY_VERBOSE = false;
 
         // timer for performance analysis
         long[] times = new long[PATTERNS.length];
         long timer;
 
-        if (DEBUG_ALL) {
+        if (TEST_ALL) {
             // It may take about an hour for testing all locales
             LOCALES = ULocale.getAvailableLocales();
         } else if (getInclusion() > 5) {
@@ -203,11 +257,12 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
         } else {
             LOCALES = new ULocale[] {
                 new ULocale("en"),
-                new ULocale("en_CA"),
-                new ULocale("fr")
             };
         }
 
+        SimpleDateFormat sdfGMT = new SimpleDateFormat(BASEPATTERN);
+        sdfGMT.setTimeZone(TimeZone.getTimeZone("Etc/GMT"));
+
         long testCounts = 0;
         long[] testTimes = new long[4];
         boolean[] expectedRoundTrip = new boolean[4];
@@ -221,16 +276,20 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
 
                 String[] ids = TimeZone.getAvailableIDs();
                 for (int zidx = 0; zidx < ids.length; zidx++) {
-                    if(!ids[zidx].equals(ZoneMeta.getCanonicalID(ids[zidx]))) {
+                    String id = TimeZone.getCanonicalID(ids[zidx]);
+                    if (id == null || !id.equals(ids[zidx])) {
                         // Skip aliases
                         continue;
                     }
-                    BasicTimeZone tz = (BasicTimeZone)TimeZone.getTimeZone(ids[zidx]);
+                    BasicTimeZone btz = (BasicTimeZone)TimeZone.getTimeZone(ids[zidx], TimeZone.TIMEZONE_ICU);
+                    TimeZone tz = TimeZone.getTimeZone(ids[zidx]);
                     sdf.setTimeZone(tz);
 
                     long t = START_TIME;
                     TimeZoneTransition tzt = null;
                     boolean middle = true;
+                    boolean isJdkTimeZone = (TimeZone.getDefaultTimeZoneType() == TimeZone.TIMEZONE_JDK);
+testTransitions:
                     while (t < END_TIME) {
                         if (tzt == null) {
                             testTimes[0] = t;
@@ -275,6 +334,7 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
                                         .append(", locale=").append(LOCALES[locidx])
                                         .append(", pattern=").append(PATTERNS[patidx])
                                         .append(", text=").append(text)
+                                        .append(", gmt=").append(sdfGMT.format(new Date(testTimes[testidx])))
                                         .append(", time=").append(testTimes[testidx])
                                         .append(", restime=").append(restime)
                                         .append(", diff=").append(restime - testTimes[testidx]);
@@ -289,10 +349,24 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
                             }
                             times[patidx] += System.currentTimeMillis() - timer;
                         }
-                        tzt = tz.getNextTransition(t, false);
-                        if (tzt == null) {
-                            break;
+
+                        long tmpt = t;
+                        while (tmpt < END_TIME) {
+                            tzt = btz.getNextTransition(tmpt, false);
+                            if (tzt == null) {
+                                break testTransitions;
+                            }
+                            // JDK time zone data may be out of synch
+                            if (!isJdkTimeZone || (tz.getOffset(tzt.getTime() - 1) != tz.getOffset(tzt.getTime()))) {
+                                break;
+                            }
+                            tmpt = tzt.getTime();
                         }
+                        tzt = btz.getNextTransition(t, false);
+                        if (tzt == null) {
+                            break testTransitions;
+                        }
+
                         if (middle) {
                             // Test the date in the middle of two transitions.
                             t += (tzt.getTime() - t)/2;
