@@ -1,12 +1,13 @@
+//##header J2SE15
 /*
 **********************************************************************
-* Copyright (c) 2002-2004, International Business Machines
-* Corporation and others.  All Rights Reserved.
+* Copyright (c) 2002-2007, International Business Machines           *
+* Corporation and others.  All Rights Reserved.                      *
 **********************************************************************
 */
 package com.ibm.icu.dev.test.perf;
 
-import com.ibm.icu.text.*;
+import com.ibm.icu.text.Normalizer;
 
 public class NormalizerPerformanceTest extends PerfTest {
     
@@ -14,17 +15,12 @@ public class NormalizerPerformanceTest extends PerfTest {
     String[] NFCFileLines;
     String[] fileLines;
     
+    
     public static void main(String[] args) throws Exception {
         new NormalizerPerformanceTest().run(args);
     }
     
     protected void setup(String[] args) {
-        if(bulk_mode == line_mode){
-            printUsage();
-        }
-        if (fileName.equalsIgnoreCase("")){
-            printUsage();
-        }
         fileLines = readLines(fileName, encoding, bulk_mode);
         NFDFileLines = normalizeInput(fileLines, Normalizer.NFD);
         NFCFileLines = normalizeInput(fileLines, Normalizer.NFC);
@@ -139,21 +135,19 @@ public class NormalizerPerformanceTest extends PerfTest {
             }
         };
     }
-/** I really wish there was conditional compilation in Java      
-        // Test NFC Performance
+
+    // Test NFC Performance
     PerfTest.Function TestJDK_NFC_NFD_Text() {
         return new PerfTest.Function() {
             public void call() {
-                for (int i = 0; i < NFDFileLines.length; i++) {
-                    String nfc = sun.text.Normalizer.normalize(NFDFileLines[i], sun.text.Normalizer.COMPOSE,0);
-                }
+                for (int i = 0; i < NFDFileLines.length; i++)
+                    normalizerTest(NFDFileLines[i], true);
             }
             
             public long getOperationsPerIteration() {
                 int totalChars = 0;
-                for (int i = 0; i < NFDFileLines.length; i++) {
+                for (int i = 0; i < NFDFileLines.length; i++)
                     totalChars = totalChars + NFDFileLines[i].length();
-                }
                 return totalChars;
             }
         };
@@ -162,16 +156,14 @@ public class NormalizerPerformanceTest extends PerfTest {
     PerfTest.Function TestJDK_NFC_NFC_Text() {
         return new PerfTest.Function() {
             public void call() {
-                for (int i = 0; i < NFCFileLines.length; i++) {
-                    String nfc = sun.text.Normalizer.normalize(NFCFileLines[i], sun.text.Normalizer.COMPOSE,0);
-                }
+                for (int i = 0; i < NFCFileLines.length; i++)
+                    normalizerTest(NFCFileLines[i], true);
             }
             
             public long getOperationsPerIteration() {
                 int totalChars = 0;
-                for (int i = 0; i < NFCFileLines.length; i++) {
+                for (int i = 0; i < NFCFileLines.length; i++)
                     totalChars = totalChars + NFCFileLines[i].length();
-                }
                 return totalChars;
             }
         };
@@ -180,16 +172,14 @@ public class NormalizerPerformanceTest extends PerfTest {
     PerfTest.Function TestJDK_NFC_Orig_Text() {
         return new PerfTest.Function() {
             public void call() {
-                for (int i = 0; i < fileLines.length; i++) {
-                    String nfc = sun.text.Normalizer.normalize(fileLines[i], sun.text.Normalizer.COMPOSE,0);
-                }
+                for (int i = 0; i < fileLines.length; i++)
+                    normalizerTest(fileLines[i], true);
             }
             
             public long getOperationsPerIteration() {
                 int totalChars = 0;
-                for (int i = 0; i < fileLines.length; i++) {
+                for (int i = 0; i < fileLines.length; i++)
                     totalChars = totalChars + fileLines[i].length();
-                }
                 return totalChars;
             }
         };
@@ -199,16 +189,14 @@ public class NormalizerPerformanceTest extends PerfTest {
     PerfTest.Function TestJDK_NFD_NFD_Text() {
         return new PerfTest.Function() {
             public void call() {
-                for (int i = 0; i < NFDFileLines.length; i++) {
-                    String nfc = sun.text.Normalizer.normalize(NFDFileLines[i], sun.text.Normalizer.DECOMP,0);
-                }
+                for (int i = 0; i < NFDFileLines.length; i++)
+                    normalizerTest(NFDFileLines[i], false);
             }
             
             public long getOperationsPerIteration() {
                 int totalChars = 0;
-                for (int i = 0; i < NFDFileLines.length; i++) {
+                for (int i = 0; i < NFDFileLines.length; i++)
                     totalChars = totalChars + NFDFileLines[i].length();
-                }
                 return totalChars;
             }
         };
@@ -217,16 +205,14 @@ public class NormalizerPerformanceTest extends PerfTest {
     PerfTest.Function TestJDK_NFD_NFC_Text() {
         return new PerfTest.Function() {
             public void call() {
-                for (int i = 0; i < NFCFileLines.length; i++) {
-                    String nfc = sun.text.Normalizer.normalize(NFCFileLines[i], sun.text.Normalizer.DECOMP,0);
-                }
+                for (int i = 0; i < NFCFileLines.length; i++)
+                    normalizerTest(NFCFileLines[i], false);
             }
             
             public long getOperationsPerIteration() {
                 int totalChars = 0;
-                for (int i = 0; i < NFCFileLines.length; i++) {
+                for (int i = 0; i < NFCFileLines.length; i++)
                     totalChars = totalChars + NFCFileLines[i].length();
-                }
                 return totalChars;
             }
         };
@@ -235,16 +221,14 @@ public class NormalizerPerformanceTest extends PerfTest {
     PerfTest.Function TestJDK_NFD_Orig_Text() {
         return new PerfTest.Function() {
             public void call() {
-                for (int i = 0; i < fileLines.length; i++) {
-                    String nfc = sun.text.Normalizer.normalize(fileLines[i], sun.text.Normalizer.DECOMP,0);
-                }
+                for (int i = 0; i < fileLines.length; i++)
+                    normalizerTest(fileLines[i], false);
             }
             
             public long getOperationsPerIteration() {
                 int totalChars = 0;
-                for (int i = 0; i < fileLines.length; i++) {
+                for (int i = 0; i < fileLines.length; i++)
                     totalChars = totalChars + fileLines[i].length();
-                }
                 return totalChars;
             }
         };
@@ -254,7 +238,7 @@ public class NormalizerPerformanceTest extends PerfTest {
         return new PerfTest.Function() {
             public void call() {
                 for (int i = 0; i < NFDFileLines.length; i++) {
-                    String nfc = Normalizer.normalize(NFDFileLines[i], Normalizer.FCD);
+                    Normalizer.normalize(NFDFileLines[i], Normalizer.FCD);
                 }
             }
             
@@ -267,7 +251,7 @@ public class NormalizerPerformanceTest extends PerfTest {
             }
         };
     }
-**/    
+
     PerfTest.Function TestICU_FCD_NFC_Text() {
         return new PerfTest.Function() {
             public void call() {
@@ -630,7 +614,8 @@ public class NormalizerPerformanceTest extends PerfTest {
         };
     }
       
-    private void printUsage() {
+    /*
+      private void printUsage() {
         System.out.println("Usage: " + this.getClass().getName() + " [OPTIONS] fileName\n"
                             + "\t-f or --fileName  \tfile to be used as test data\n"
                             + "\t-s or --sourceDir \tsource directory for files followed by path\n"
@@ -640,6 +625,8 @@ public class NormalizerPerformanceTest extends PerfTest {
             );
         System.exit(1);
     }
+    */
+    
     String[] normalizeInput(String[] src, Normalizer.Mode mode) {
         String[] dest = new String[src.length];
         for (int i = 0; i < src.length; i++) {
@@ -648,4 +635,72 @@ public class NormalizerPerformanceTest extends PerfTest {
         
         return dest;
     }
+    
+    /*
+    void normalizerInit(boolean compose) {
+        Class normalizer;
+        boolean sun;
+        
+        try {
+            normalizer = Class.forName("java.text.Normalizer");
+            sun = false;
+        } catch (ClassNotFoundException ex) {
+            try {
+                normalizer = Class.forName("sun.text.Normalizer");
+                sun = true;
+            } catch (ClassNotFoundException ex2) {
+                throw new RuntimeException(
+                        "Could not find sun.text.Normalizer nor java.text.Normalizer and their required subclasses");
+            }
+        }
+        
+        try {
+            if (sun) {
+                normalizerArgs = new Object[] { null, null, new Integer(0) };
+                normalizerArgs[1] = normalizer.getField(compose ? "COMPOSE" : "DECOMP").get(null);
+                normalizerMethod = normalizer.getMethod("normalize", new Class[] { String.class, normalizerArgs[1].getClass(), int.class });
+                // sun.text.Normalizer.normalize(line, compose
+                //   ? sun.text.Normalizer.COMPOSE
+                //   : sun.text.Normalizer.DECOMP, 0);
+            } else {
+                normalizerArgs = new Object[] { null, null };
+                normalizerArgs[1] = Class.forName("java.text.Normalizer$Form").getField(compose ? "NFC" : "NFD").get(null);
+                normalizerMethod = normalizer.getMethod("normalize", new Class[] { CharSequence.class, normalizerArgs[1].getClass()});
+                // java.text.Normalizer.normalize(line, compose
+                //   ? java.text.Normalizer.Form.NFC
+                //   : java.text.Normalizer.Form.NFD);
+            }
+        } catch (Exception ex) {
+            ex.printStackTrace();
+            throw new RuntimeException("Reflection error -- could not load the JDK normalizer (" + normalizer.getName() + ")");
+        }
+    }
+    
+    void normalizerTest(String line) {
+        try {
+            normalizerArgs[0] = line;
+            normalizerMethod.invoke(line, normalizerArgs);
+        } catch (Exception ex) {
+            if (ex instanceof InvocationTargetException) {
+                Throwable cause = ex.getCause();
+                cause.printStackTrace();
+                throw new RuntimeException(cause.getMessage());
+            } else {
+                throw new RuntimeException("Reflection error -- could not run the JDK normalizer");
+            }
+        }
+    }
+    */
+
+    void normalizerTest(String line, boolean compose) {
+//#if defined(FOUNDATION10) || defined(J2SE13) || defined(J2SE14) || defined(J2SE15)
+ sun.text.Normalizer.normalize(line, compose
+      ? sun.text.Normalizer.COMPOSE
+      : sun.text.Normalizer.DECOMP, 0);
+//#else
+//## java.text.Normalizer.normalize(line, compose
+//##      ? java.text.Normalizer.Form.NFC
+//##      : java.text.Normalizer.Form.NFD);
+//#endif
+    }
 }
