@@ -1,7 +1,7 @@
 //##header J2SE15
 /**
  *******************************************************************************
- * Copyright (C) 2001-2007, International Business Machines Corporation and    *
+ * Copyright (C) 2001-2008, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -42,8 +42,8 @@ public final class ICUResourceBundleTest extends TestFmwk {
     }
     public void TestGetResources(){
         try{
-        	// It does not work well in eclipse plug-in test because of class loader configuration??
-        	// For now, specify resource path explicitly in this test case
+            // It does not work well in eclipse plug-in test because of class loader configuration??
+            // For now, specify resource path explicitly in this test case
             //Enumeration en = testLoader.getResources("META-INF");
             Enumeration en = testLoader.getResources("com.ibm.icu.dev.data");
             for(;en.hasMoreElements();) {
@@ -454,6 +454,7 @@ public final class ICUResourceBundleTest extends TestFmwk {
     }
 
     public void TestAliases(){
+/*
        String simpleAlias   = "Open";
 
        UResourceBundle rb = (UResourceBundle)UResourceBundle.getBundleInstance("com/ibm/icu/dev/data/testdata","testaliases", testLoader);
@@ -574,7 +575,7 @@ public final class ICUResourceBundleTest extends TestFmwk {
         // should not get an exception
         rb = (UResourceBundle) UResourceBundle.getBundleInstance(ICUResourceBundle.ICU_RBNF_BASE_NAME,"fr_BE");
         String str = rb.getString("SpelloutRules");
-        if(str !=null || str.length()>0){
+        if(str !=null && str.length()>0){
             logln("Alias mechanism works");
         }else{
             errln("Alias mechanism failed for fr_BE SpelloutRules");
@@ -590,7 +591,7 @@ public final class ICUResourceBundleTest extends TestFmwk {
         }else{
             errln("Did not get the expected object for collations");
         }
-
+*/
     }
     public void TestAlias(){
         logln("Testing %%ALIAS");
@@ -627,19 +628,20 @@ public final class ICUResourceBundleTest extends TestFmwk {
         }
     }
     public void TestCircularAliases(){
-        try{
-            UResourceBundle rb = (UResourceBundle)UResourceBundle.getBundleInstance("com/ibm/icu/dev/data/testdata","testaliases",testLoader);
-            UResourceBundle sub = rb.get("aaa");
-            String s1 = sub.getString();
-            if(s1!=null){
-                errln("Did not get the expected exception");
-            }
-        }catch(IllegalArgumentException ex){
-            logln("got expected exception for circular references");
-        }
-        catch (MissingResourceException ex) {
-            warnln("could not load resource data: " + ex.getMessage());
-        }
+// Aliases no longer supported
+//        try{
+//            UResourceBundle rb = (UResourceBundle)UResourceBundle.getBundleInstance("com/ibm/icu/dev/data/testdata","testaliases",testLoader);
+//            UResourceBundle sub = rb.get("aaa");
+//            String s1 = sub.getString();
+//            if(s1!=null){
+//                errln("Did not get the expected exception");
+//            }
+//        }catch(IllegalArgumentException ex){
+//            logln("got expected exception for circular references");
+//        }
+//        catch (MissingResourceException ex) {
+//            warnln("could not load resource data: " + ex.getMessage());
+//        }
     }
 
     public void TestGetWithFallback(){
@@ -698,6 +700,8 @@ public final class ICUResourceBundleTest extends TestFmwk {
     private static final String COLLATION_KEYWORD = "collation";
     private static final String DEFAULT_NAME = "default";
     private static final String STANDARD_NAME = "standard";
+    private static final String CALENDAR_RESNAME = "calendar";
+    private static final String CALENDAR_KEYWORD = "calendar";
 
     public void TestKeywordValues(){
         String kwVals[];
@@ -762,87 +766,108 @@ public final class ICUResourceBundleTest extends TestFmwk {
     }
 
     public void TestFunctionalEquivalent(){
-       String[] testCases = {
-              //              avail   locale            equiv
-              "f",    "de_US_CALIFORNIA",               "de",
-              "f",    "zh_TW@collation=stroke",         "zh@collation=stroke", /* alias of zh_Hant_TW */
-              "t",    "zh_Hant_TW@collation=stroke",    "zh@collation=stroke",
-              "f",    "de_CN@collation=pinyin",         "de",
-              "t",    "zh@collation=pinyin",            "zh",
-              "f",    "zh_CN@collation=pinyin",         "zh", /* alias of zh_Hans_CN */
-              "t",    "zh_Hans_CN@collation=pinyin",    "zh",
-              "f",    "zh_HK@collation=pinyin",         "zh", /* alias of zh_Hant_HK */
-              "t",    "zh_Hant_HK@collation=pinyin",    "zh",
-              "f",    "zh_HK@collation=stroke",         "zh@collation=stroke", /* alias of zh_Hant_HK */
-              "t",    "zh_Hant_HK@collation=stroke",    "zh@collation=stroke",
-              "f",    "zh_HK",                          "zh@collation=stroke", /* alias of zh_Hant_HK */
-              "t",    "zh_Hant_HK",                     "zh@collation=stroke",
-              "f",    "zh_MO",                          "zh@collation=stroke", /* alias of zh_Hant_MO */
-              "t",    "zh_Hant_MO",                     "zh@collation=stroke",
-              "f",    "zh_TW_STROKE",                   "zh@collation=stroke",
-              "f",    "zh_TW_STROKE@collation=big5han", "zh@collation=big5han",
-              "f",    "de_CN@calendar=japanese",        "de",
-              "t",    "de@calendar=japanese",           "de",
-              "f",    "zh_TW@collation=big5han",        "zh@collation=big5han", /* alias of zh_Hant_TW */
-              "t",    "zh_Hant_TW@collation=big5han",   "zh@collation=big5han",
-              "f",    "zh_TW@collation=gb2312han",      "zh@collation=gb2312han", /* alias of zh_Hant_TW */
-              "t",    "zh_Hant_TW@collation=gb2312han", "zh@collation=gb2312han",
-              "f",    "zh_CN@collation=big5han",        "zh@collation=big5han", /* alias of zh_Hans_CN */
-              "t",    "zh_Hans_CN@collation=big5han",   "zh@collation=big5han",
-              "f",    "zh_CN@collation=gb2312han",      "zh@collation=gb2312han", /* alias of zh_Hans_CN */
-              "t",    "zh_Hans_CN@collation=gb2312han", "zh@collation=gb2312han",
-              "t",    "zh@collation=big5han",           "zh@collation=big5han",
-              "t",    "zh@collation=gb2312han",         "zh@collation=gb2312han",
-              "t",    "hi_IN@collation=direct",         "hi@collation=direct",
-              "t",    "hi@collation=standard",          "hi",
-              "t",    "hi@collation=direct",            "hi@collation=direct",
-              "f",    "hi_AU@collation=direct;currency=CHF;calendar=buddhist",      "hi@collation=direct",
-              "f",    "hi_AU@collation=standard;currency=CHF;calendar=buddhist",    "hi",
-              "t",    "de_DE@collation=pinyin",         "de", /* bug 4582 tests */
-              "f",    "de_DE_BONN@collation=pinyin",    "de",
-              "t",    "nl",                             "root",
-              "t",    "nl_NL",                          "root",
-              "f",    "nl_NL_EEXT",                     "root",
-              "t",    "nl@collation=stroke",            "root",
-              "t",    "nl_NL@collation=stroke",         "root",
-              "f",    "nl_NL_EEXT@collation=stroke",    "root",	  
-           };
+       String[] collCases = {
+       //  avail   locale                               equiv
+           "f",     "de_US_CALIFORNIA",                 "de",
+           "f",     "zh_TW@collation=stroke",           "zh@collation=stroke", /* alias of zh_Hant_TW */
+           "t",     "zh_Hant_TW@collation=stroke",      "zh@collation=stroke",
+           "f",     "de_CN@collation=pinyin",           "de",
+           "t",     "zh@collation=pinyin",              "zh",
+           "f",     "zh_CN@collation=pinyin",           "zh", /* alias of zh_Hans_CN */
+           "t",     "zh_Hans_CN@collation=pinyin",      "zh",
+           "f",     "zh_HK@collation=pinyin",           "zh", /* alias of zh_Hant_HK */
+           "t",     "zh_Hant_HK@collation=pinyin",      "zh",
+           "f",     "zh_HK@collation=stroke",           "zh@collation=stroke", /* alias of zh_Hant_HK */
+           "t",     "zh_Hant_HK@collation=stroke",      "zh@collation=stroke",
+           "f",     "zh_HK",                            "zh@collation=stroke", /* alias of zh_Hant_HK */
+           "t",     "zh_Hant_HK",                       "zh@collation=stroke",
+           "f",     "zh_MO",                            "zh@collation=stroke", /* alias of zh_Hant_MO */
+           "t",     "zh_Hant_MO",                       "zh@collation=stroke",
+           "f",     "zh_TW_STROKE",                     "zh@collation=stroke",
+           "f",     "zh_TW_STROKE@collation=big5han",   "zh@collation=big5han",
+           "f",     "de_CN@calendar=japanese",          "de",
+           "t",     "de@calendar=japanese",             "de",
+           "f",     "zh_TW@collation=big5han",          "zh@collation=big5han", /* alias of zh_Hant_TW */
+           "t",     "zh_Hant_TW@collation=big5han",     "zh@collation=big5han",
+           "f",     "zh_TW@collation=gb2312han",        "zh@collation=gb2312han", /* alias of zh_Hant_TW */
+           "t",     "zh_Hant_TW@collation=gb2312han",   "zh@collation=gb2312han",
+           "f",     "zh_CN@collation=big5han",          "zh@collation=big5han", /* alias of zh_Hans_CN */
+           "t",     "zh_Hans_CN@collation=big5han",     "zh@collation=big5han",
+           "f",     "zh_CN@collation=gb2312han",        "zh@collation=gb2312han", /* alias of zh_Hans_CN */
+           "t",     "zh_Hans_CN@collation=gb2312han",   "zh@collation=gb2312han",
+           "t",     "zh@collation=big5han",             "zh@collation=big5han",
+           "t",     "zh@collation=gb2312han",           "zh@collation=gb2312han",
+           "t",     "hi_IN@collation=direct",           "hi@collation=direct",
+           "t",     "hi@collation=standard",            "hi",
+           "t",     "hi@collation=direct",              "hi@collation=direct",
+           "f",     "hi_AU@collation=direct;currency=CHF;calendar=buddhist",    "hi@collation=direct",
+           "f",     "hi_AU@collation=standard;currency=CHF;calendar=buddhist",  "hi",
+           "t",     "de_DE@collation=pinyin",           "de", /* bug 4582 tests */
+           "f",     "de_DE_BONN@collation=pinyin",      "de",
+           "t",     "nl",                               "root",
+           "t",     "nl_NL",                            "root",
+           "f",     "nl_NL_EEXT",                       "root",
+           "t",     "nl@collation=stroke",              "root",
+           "t",     "nl_NL@collation=stroke",           "root",
+           "f",     "nl_NL_EEXT@collation=stroke",      "root",
+       };
 
-       //String F_STR = "f";
-       String T_STR = "t";
-       boolean isAvail[] = new boolean[1];
-       int i;
+       String[] calCases = {
+       //  avail    locale                              equiv
+           "t",     "en_US_POSIX",                      "en_US@calendar=gregorian",
+           "f",     "ja_JP_TOKYO",                      "ja_JP@calendar=gregorian",
+           "f",     "ja_JP_TOKYO@calendar=japanese",    "ja@calendar=japanese",
+           "t",     "sr@calendar=gregorian",            "sr@calendar=gregorian",
+           "t",     "en",                               "en@calendar=gregorian",
+           "t",     "th_TH",                            "th@calendar=buddhist",
+           "t",     "th_TH@calendar=gregorian",         "th_TH@calendar=gregorian",
+           "f",     "th_TH_Bangkok",                    "th@calendar=buddhist",
+       };
 
-       logln("Testing functional equivalents...");
-       for(i=0;i<testCases.length;i+=3) {
-           boolean expectAvail = T_STR.equals(testCases[i+0]);
-           ULocale inLocale = new ULocale(testCases[i+1]);
-           ULocale expectLocale = new ULocale(testCases[i+2]);
+       logln("Testing functional equivalents for collation...");
+       getFunctionalEquivalentTestCases(ICUResourceBundle.ICU_COLLATION_BASE_NAME,
+               COLLATION_RESNAME, COLLATION_KEYWORD, true, collCases);
 
-           logln(new Integer(i/3).toString() + ": " + new Boolean(expectAvail).toString() + "\t\t" +
-                   inLocale.toString() + "\t\t" + expectLocale.toString());
-
-           ULocale equivLocale = ICUResourceBundle.getFunctionalEquivalent(ICUResourceBundle.ICU_COLLATION_BASE_NAME,COLLATION_RESNAME,
-                   COLLATION_KEYWORD, inLocale, isAvail);
-           boolean gotAvail = isAvail[0];
-
-           if((gotAvail!=expectAvail) || !equivLocale.equals(expectLocale)) {
-               errln(new Integer(i/3).toString() + ":  Error, expected  Equiv=" + new Boolean(expectAvail).toString() + "\t\t" +
-                       inLocale.toString() + "\t\t--> " + expectLocale.toString() + ",  but got " + new Boolean(gotAvail).toString() + " " +
-                       equivLocale.toString());
-           }
-       }
+       logln("Testing functional equivalents for calenadr...");
+       getFunctionalEquivalentTestCases(ICUResourceBundle.ICU_BASE_NAME,
+               CALENDAR_RESNAME, CALENDAR_KEYWORD, false, calCases);
 
        logln("Testing error conditions:");
        try {
            ICUResourceBundle.getFunctionalEquivalent(ICUResourceBundle.ICU_COLLATION_BASE_NAME, "calendar",
-              "calendar", new ULocale("ar_EG@calendar=islamic"), new boolean[1]);
+              "calendar", new ULocale("ar_EG@calendar=islamic"), new boolean[1], true);
            errln("Err: expected MissingResourceException");
        } catch ( MissingResourceException t ) {
            logln("expected MissingResourceException caught (PASS): " + t.toString());
        }
     }
 
+    private void getFunctionalEquivalentTestCases(String path, String resName, String keyword,
+            boolean truncate, String[] testCases) {
+        //String F_STR = "f";
+        String T_STR = "t";
+        boolean isAvail[] = new boolean[1];
+
+        logln("Testing functional equivalents...");
+        for(int i = 0; i < testCases.length ;i+=3) {
+            boolean expectAvail = T_STR.equals(testCases[i+0]);
+            ULocale inLocale = new ULocale(testCases[i+1]);
+            ULocale expectLocale = new ULocale(testCases[i+2]);
+
+            logln(new Integer(i/3).toString() + ": " + new Boolean(expectAvail).toString() + "\t\t" +
+                    inLocale.toString() + "\t\t" + expectLocale.toString());
+
+            ULocale equivLocale = ICUResourceBundle.getFunctionalEquivalent(path, resName, keyword, inLocale, isAvail, truncate);
+            boolean gotAvail = isAvail[0];
+
+            if((gotAvail != expectAvail) || !equivLocale.equals(expectLocale)) {
+                errln(new Integer(i/3).toString() + ":  Error, expected  Equiv=" + new Boolean(expectAvail).toString() + "\t\t" +
+                        inLocale.toString() + "\t\t--> " + expectLocale.toString() + ",  but got " + new Boolean(gotAvail).toString() + " " +
+                        equivLocale.toString());
+            }
+        }
+    }
+
     public void TestNorwegian(){
         try{
             UResourceBundle rb = UResourceBundle.getBundleInstance(ICUResourceBundle.ICU_BASE_NAME, "no_NO_NY");
@@ -997,7 +1022,7 @@ public final class ICUResourceBundleTest extends TestFmwk {
             protected UResourceBundle getParent() {return null;}
             protected void setLoadingStatus(int newStatus) {}
             public Enumeration getKeys() {return null;}
-            protected Object handleGetObject(String key) {return null;}
+            protected Object handleGetObject(String aKey) {return null;}
         }
         Stub stub = new Stub();
         
