@@ -1,14 +1,13 @@
 //##header J2SE15
 /*
  *******************************************************************************
- * Copyright (C) 1996-2007, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2008, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
 
 package com.ibm.icu.text;
 
-//import com.ibm.icu.impl.ICULocaleData;
 import com.ibm.icu.impl.ICUDebug;
 import com.ibm.icu.impl.ICUResourceBundle;
 import com.ibm.icu.impl.UCharacterProperty;
@@ -26,8 +25,6 @@ import java.util.Map;
 import java.util.MissingResourceException;
 import java.util.Set;
 
-//import java.util.ResourceBundle;
-
 
 /**
  * <p>A class that formats numbers according to a set of rules. This number formatter is
@@ -546,6 +543,13 @@ public class RuleBasedNumberFormat extends NumberFormat {
      * filled in if the rule set never uses a DecimalFormat pattern.
      */
     private transient DecimalFormatSymbols decimalFormatSymbols = null;
+    
+    /**
+     * The NumberFormat used when lenient parsing numbers.  This needs to reflect
+     * the locale.  This is lazy-evaluated, like decimalFormatSymbols.  It is
+     * here so it can be shared by different NFSubstitutions.
+     */
+    private transient DecimalFormat decimalFormat = null;
 
     /**
      * Flag specifying whether lenient parse mode is on or off.  Off by default.
@@ -876,6 +880,7 @@ public class RuleBasedNumberFormat extends NumberFormat {
         defaultRuleSet = temp.defaultRuleSet;
         publicRuleSetNames = temp.publicRuleSetNames;
         decimalFormatSymbols = temp.decimalFormatSymbols;
+        decimalFormat = temp.decimalFormat;
         locale = temp.locale;
     }
 
@@ -913,9 +918,9 @@ public class RuleBasedNumberFormat extends NumberFormat {
         return null;
     }
 
-    private String[] getNameListForLocale(ULocale locale) {
-        if (locale != null && ruleSetDisplayNames != null) {
-            String[] localeNames = { locale.getBaseName(), ULocale.getDefault().getBaseName() };
+    private String[] getNameListForLocale(ULocale loc) {
+        if (loc != null && ruleSetDisplayNames != null) {
+            String[] localeNames = { loc.getBaseName(), ULocale.getDefault().getBaseName() };
             for (int i = 0; i < localeNames.length; ++i) {
                 String lname = localeNames[i];
                 while (lname.length() > 0) {
@@ -940,8 +945,8 @@ public class RuleBasedNumberFormat extends NumberFormat {
      * @see #getRuleSetNames
      * @stable ICU 3.2
      */
-    public String[] getRuleSetDisplayNames(ULocale locale) {
-        String[] names = getNameListForLocale(locale);
+    public String[] getRuleSetDisplayNames(ULocale loc) {
+        String[] names = getNameListForLocale(loc);
         if (names != null) {
             return (String[])names.clone();
         }
@@ -971,11 +976,11 @@ public class RuleBasedNumberFormat extends NumberFormat {
      * @throws IllegalArgumentException if ruleSetName is not a valid rule set name for this format
      * @stable ICU 3.2
      */
-    public String getRuleSetDisplayName(String ruleSetName, ULocale locale) {
+    public String getRuleSetDisplayName(String ruleSetName, ULocale loc) {
         String[] rsnames = publicRuleSetNames;
         for (int ix = 0; ix < rsnames.length; ++ix) {
             if (rsnames[ix].equals(ruleSetName)) {
-                String[] names = getNameListForLocale(locale);
+                String[] names = getNameListForLocale(loc);
                 if (names != null) {
                     return names[ix];
                 }
@@ -1083,7 +1088,7 @@ public class RuleBasedNumberFormat extends NumberFormat {
         return format(new com.ibm.icu.math.BigDecimal(number), toAppendTo, pos);
     }
 
-//#if defined(FOUNDATION10) || defined(J2SE13)
+//#if defined(FOUNDATION10)
 //#else
     /**
      * <strong><font face=helvetica color=red>NEW</font></strong>
@@ -1344,6 +1349,13 @@ public class RuleBasedNumberFormat extends NumberFormat {
         }
         return decimalFormatSymbols;
     }
+    
+    DecimalFormat getDecimalFormat() {
+        if (decimalFormat == null) {
+            decimalFormat = (DecimalFormat)NumberFormat.getInstance(locale);
+        }
+        return decimalFormat;
+    }
 
     //-----------------------------------------------------------------------
     // construction implementation
@@ -1527,14 +1539,14 @@ public class RuleBasedNumberFormat extends NumberFormat {
             Map m = new HashMap();
             for (int i = 1; i < localizations.length; ++i) {
                 String[] data = localizations[i];
-                String locale = data[0];
+                String loc = data[0];
                 String[] names = new String[data.length-1];
                 if (names.length != publicRuleSetNames.length) {
                     throw new IllegalArgumentException("public name length: " + publicRuleSetNames.length +
                                                        " != localized names[" + i + "] length: " + names.length);
                 }
                 System.arraycopy(data, 1, names, 0, names.length);
-                m.put(locale, names);
+                m.put(loc, names);
             }
 
             if (!m.isEmpty()) {
