@@ -1,6 +1,6 @@
 /*
  *******************************************************************************
- * Copyright (C) 2007, International Business Machines Corporation and         *
+ * Copyright (C) 2007-2008, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -17,6 +17,7 @@ import java.util.List;
 import java.util.MissingResourceException;
 
 import com.ibm.icu.impl.Grego;
+import com.ibm.icu.util.DateTimeRule;
 
 /**
  * <code>VTimeZone</code> is a class implementing RFC2445 VTIMEZONE.  You can create a
@@ -30,8 +31,7 @@ import com.ibm.icu.impl.Grego;
  * decode or encode Non-ASCII text.  Methods reading/writing VTIMEZONE data in this class
  * do nothing with MIME encoding.
  * 
- * @draft ICU 3.8
- * @provisional This API might change or be removed in a future release.
+ * @stable ICU 3.8
  */
 public class VTimeZone extends BasicTimeZone {
 
@@ -44,12 +44,11 @@ public class VTimeZone extends BasicTimeZone {
      * @return A <code>VTimeZone</code> initialized by the time zone ID, or null
      * when the ID is unknown.
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public static VTimeZone create(String tzid) {
         VTimeZone vtz = new VTimeZone();
-        vtz.tz = (BasicTimeZone)TimeZone.getTimeZone(tzid);
+        vtz.tz = (BasicTimeZone)TimeZone.getTimeZone(tzid, TimeZone.TIMEZONE_ICU);
         vtz.olsonzid = vtz.tz.getID();
         vtz.setID(tzid);
 
@@ -63,8 +62,7 @@ public class VTimeZone extends BasicTimeZone {
      * @return A <code>VTimeZone</code> initialized by the VTIMEZONE data or
      * null if failed to load the rule from the VTIMEZONE data.
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public static VTimeZone create(Reader reader) {
         VTimeZone vtz = new VTimeZone();
@@ -76,8 +74,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public int getOffset(int era, int year, int month, int day, int dayOfWeek,
             int milliseconds) {
@@ -86,8 +83,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public void getOffset(long date, boolean local, int[] offsets) {
         tz.getOffset(date, local, offsets);
@@ -105,8 +101,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public int getRawOffset() {
         return tz.getRawOffset();
@@ -114,8 +109,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public boolean inDaylightTime(Date date) {
         return tz.inDaylightTime(date);
@@ -123,8 +117,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public void setRawOffset(int offsetMillis) {
         tz.setRawOffset(offsetMillis);
@@ -132,8 +125,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public boolean useDaylightTime() {
         return tz.useDaylightTime();
@@ -141,8 +133,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public boolean hasSameRules(TimeZone other) {
         return tz.hasSameRules(other);
@@ -155,8 +146,7 @@ public class VTimeZone extends BasicTimeZone {
      * 
      * @return The RFC2445 TZURL property value
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public String getTZURL() {
         return tzurl;
@@ -167,8 +157,7 @@ public class VTimeZone extends BasicTimeZone {
      * 
      * @param url The TZURL property value.
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public void setTZURL(String url) {
         tzurl = url;
@@ -181,8 +170,7 @@ public class VTimeZone extends BasicTimeZone {
      * 
      * @return The Date represents the RFC2445 LAST-MODIFIED date.
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public Date getLastModified() {
         return lastmod;
@@ -193,8 +181,7 @@ public class VTimeZone extends BasicTimeZone {
      * 
      * @param date The <code>Date</code> object represents the date for RFC2445 LAST-MODIFIED property value.
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public void setLastModified(Date date) {
         lastmod = date;
@@ -206,8 +193,7 @@ public class VTimeZone extends BasicTimeZone {
      * @param writer A <code>Writer</code> used for the output
      * @throws IOException
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public void write(Writer writer) throws IOException {
         BufferedWriter bw = new BufferedWriter(writer);
@@ -254,8 +240,7 @@ public class VTimeZone extends BasicTimeZone {
      * 
      * @throws IOException
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public void write(Writer writer, long start) throws IOException {
         // Extract rules applicable to dates after the start time
@@ -289,8 +274,7 @@ public class VTimeZone extends BasicTimeZone {
      * 
      * @throws IOException
      * 
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public void writeSimple(Writer writer, long time) throws IOException {
         // Extract simple rules
@@ -314,8 +298,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public TimeZoneTransition getNextTransition(long base, boolean inclusive) {
         return tz.getNextTransition(base, inclusive);
@@ -323,8 +306,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public TimeZoneTransition getPreviousTransition(long base, boolean inclusive) {
         return tz.getPreviousTransition(base, inclusive);
@@ -332,8 +314,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public boolean hasEquivalentTransitions(TimeZone other, long start, long end) {
         return tz.hasEquivalentTransitions(other, start, end);
@@ -341,8 +322,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public TimeZoneRule[] getTimeZoneRules() {
         return tz.getTimeZoneRules();
@@ -350,8 +330,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public TimeZoneRule[] getTimeZoneRules(long start) {
         return tz.getTimeZoneRules(start);
@@ -359,8 +338,7 @@ public class VTimeZone extends BasicTimeZone {
 
     /**
      * {@inheritDoc}
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public Object clone() {
         VTimeZone other = (VTimeZone)super.clone();
@@ -668,7 +646,7 @@ public class VTimeZone extends BasicTimeZone {
                         // Parse TZOFFSETFROM/TZOFFSETTO
                         fromOffset = offsetStrToMillis(from);
                         toOffset = offsetStrToMillis(to);
-                        
+
                         if (dst) {
                             // If daylight, use the previous offset as rawoffset if positive
                             if (toOffset - fromOffset > 0) {
@@ -747,6 +725,75 @@ public class VTimeZone extends BasicTimeZone {
 
         // Finally, create the RuleBasedTimeZone
         RuleBasedTimeZone rbtz = new RuleBasedTimeZone(tzid, initialRule);
+
+        int finalRuleIdx = -1;
+        int finalRuleCount = 0;
+        for (int i = 0; i < rules.size(); i++) {
+            TimeZoneRule r = (TimeZoneRule)rules.get(i);
+            if (r instanceof AnnualTimeZoneRule) {
+                if (((AnnualTimeZoneRule)r).getEndYear() == AnnualTimeZoneRule.MAX_YEAR) {
+                    finalRuleCount++;
+                    finalRuleIdx = i;
+                }
+            }
+        }
+        if (finalRuleCount > 2) {
+            // Too many final rules
+            return false;
+        }
+
+        if (finalRuleCount == 1) {
+            if (rules.size() == 1) {
+                // Only one final rule, only governs the initial rule,
+                // which is already initialized, thus, we do not need to
+                // add this transition rule
+                rules.clear();
+            } else {
+                // Normalize the final rule
+                AnnualTimeZoneRule finalRule = (AnnualTimeZoneRule)rules.get(finalRuleIdx);
+                int tmpRaw = finalRule.getRawOffset();
+                int tmpDST = finalRule.getDSTSavings();
+    
+                // Find the last non-final rule
+                Date finalStart = finalRule.getFirstStart(initialRawOffset, initialDSTSavings);
+                Date start = finalStart;
+                for (int i = 0; i < rules.size(); i++) {
+                    if (finalRuleIdx == i) {
+                        continue;
+                    }
+                    TimeZoneRule r = (TimeZoneRule)rules.get(i);
+                    Date lastStart = r.getFinalStart(tmpRaw, tmpDST);
+                    if (lastStart.after(start)) {
+                        start = finalRule.getNextStart(lastStart.getTime(),
+                                r.getRawOffset(),
+                                r.getDSTSavings(),
+                                false);
+                    }
+                }
+                TimeZoneRule newRule;
+                if (start == finalStart) {
+                    // Transform this into a single transition
+                    newRule = new TimeArrayTimeZoneRule(
+                            finalRule.getName(),
+                            finalRule.getRawOffset(),
+                            finalRule.getDSTSavings(),
+                            new long[] {finalStart.getTime()},
+                            DateTimeRule.UTC_TIME);
+                } else {
+                    // Update the end year
+                    int fields[] = Grego.timeToFields(start.getTime(), null);
+                    newRule = new AnnualTimeZoneRule(
+                            finalRule.getName(),
+                            finalRule.getRawOffset(),
+                            finalRule.getDSTSavings(),
+                            finalRule.getRule(),
+                            finalRule.getStartYear(),
+                            fields[0]);
+                }
+                rules.set(finalRuleIdx, newRule);
+            }
+        }
+
         Iterator rit = rules.iterator();
         while(rit.hasNext()) {
             rbtz.addTransitionRule((TimeZoneRule)rit.next());
